/*
 * Created on May 14, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.awt.*;
import javax.swing.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * This is the window that displays the information about eShopper and eGroup
 * 
 * @author Nathan
 */
public class AboutWindow extends JFrame implements ActionListener{
	
	JPanel mainPanel;
	JPanel imagePanel;
	JPanel textPanel;
	JPanel buttonPanel;
	JButton aButton;
	
	public AboutWindow(){
		setTitle("About eShopper");
		setSize(300, 325);
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		
		imagePanel = new JPanel();
		JLabel banner = new JLabel(createImageIcon("images/banner_small.gif"));
		imagePanel.add(banner);
		
		textPanel = new JPanel();
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.Y_AXIS));
		textPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		JLabel aLabel = new JLabel("eShopper 1.0");
		aLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		textPanel.add(aLabel);
		textPanel.add(new JLabel(" "));
		textPanel.add(new JLabel("created by eGroup:"));
		textPanel.add(new JLabel(" "));
		textPanel.add(new JLabel("\tAdam Goforth"));
		textPanel.add(new JLabel("\tPam Betten"));
		textPanel.add(new JLabel("\tJim Laing"));
		textPanel.add(new JLabel("\tShazhad Qureshi"));
		textPanel.add(new JLabel("\tNathan Edwards"));
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(12,12,11,11));
		buttonPanel.add(Box.createHorizontalGlue());
		aButton = new JButton("Close");
		aButton.addActionListener(this);
		aButton.setActionCommand("Close");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		buttonPanel.add(aButton);
		
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		mainPanel.add(imagePanel, BorderLayout.NORTH);
		mainPanel.add(textPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel);
		
	}
	/**
	 * This closes the window when the user clicks the OK button.
	 */
	public void actionPerformed(ActionEvent arg0) {
		dispose();		
	}
	
	/** 
	 * This creates an ImageIcon from an image given in the path.
	 * 
	 * @param path The path of the image to be put in the ImageIcon.
	 * @return The ImageIcon, or null if the path was invalid.
	 */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = eShopper.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}
