/*
 * Created on Apr 20, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.Vector;


/**
 * This window allows the user to add a recipe or an ingredient to
 * the current list.  The user can specify a date and meal for the
 * added item, if so desired.
 * 
 * @author Nathan
 */
public class AddItemWindow extends eShopperGUI implements ListSelectionListener{
	
	JPanel mainPanel;	
	JPanel ingredientPanel;
	JPanel recipePanel;
	JPanel schedulePanel;
	JPanel datePanel;
	JPanel mealPanel;
	JPanel buttonPanel;
	JComboBox mealMenu;
	JTable calTable; 
	eShopperCalendar cal;
	JLabel monthString;
	JLabel dateString;
	JCheckBox dateAssigned;
	JTextField amountField;
	JComboBox unitBox;
	
	// constructor
	AddItemWindow(eShopperList list){
		this.list = list;
		this.database = list.getDB();
		
		setSize(350, 500);
		//setResizable(false);
		setTitle("eShopper: Add Item");
		mainPanel = new JPanel();
		mainPanel.setBorder(BorderFactory.createEmptyBorder(12,12,11,11));
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		mainPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		//make recipePanel
		recipePanel = new JPanel();
		recipePanel.setMinimumSize( new Dimension(300, 60) );
		recipePanel.setMaximumSize( new Dimension(300, 90) );
		recipePanel.setPreferredSize( new Dimension(300, 70) );
		recipePanel.setLayout( new BoxLayout(recipePanel, BoxLayout.Y_AXIS) );
		recipePanel.setBorder(BorderFactory.createEtchedBorder());
		recipePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		TitledBorder title;
		title = BorderFactory.createTitledBorder("Select a recipe to add");
		recipePanel.setBorder(title);		
		Vector recipeVector = new Vector();
		try{
			ResultSet rs = database.query("SELECT ID, Name FROM Recipe ORDER BY Name;");
			while (rs.next())recipeVector.add(new Recipe(rs.getInt(1),
													rs.getString(2)));
		} catch (SQLException se) {
			System.out.println("Unable to get recipes from database. Error:");
			se.printStackTrace();
		}
		recipesBox = new JComboBox(recipeVector);
		recipesBox.setMinimumSize( new Dimension( 250, 25) );
		recipesBox.setMaximumSize( new Dimension( 250, 25) );
		recipesBox.setPreferredSize( new Dimension( 250, 25) );
		recipePanel.add(recipesBox);
		
		// recipe button panel
		buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout(buttonPanel, BoxLayout.X_AXIS) );
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		buttonPanel.add(Box.createHorizontalGlue());
		JButton aButton = new JButton("Add");
		aButton.addActionListener(this);
		aButton.setActionCommand("addRecipe");
		buttonPanel.add(aButton);
		buttonPanel.add( Box.createRigidArea(new Dimension(10,0)) );
		aButton = new JButton("Create Recipe");
		aButton.addActionListener(this);
		aButton.setActionCommand("createRecipe");
		aButton.setPreferredSize( new Dimension( 150, 25) );
		buttonPanel.add(aButton);
		recipePanel.add(buttonPanel);
		
		//select an ingredient
		ingredientPanel = new JPanel();
		ingredientPanel.setMinimumSize( new Dimension(300, 60) );
		ingredientPanel.setMaximumSize( new Dimension(300, 90) );
		ingredientPanel.setPreferredSize( new Dimension(300, 70) );
		ingredientPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		ingredientPanel.setLayout( new BoxLayout(ingredientPanel, BoxLayout.Y_AXIS) );
		ingredientPanel.setBorder(BorderFactory.createEtchedBorder());
		
		// Reuse title from recipePanel
		title = BorderFactory.createTitledBorder("Select an ingredient to add");
		ingredientPanel.setBorder(title);
		
		// Create a panel for the ingredients controls
		JPanel setIngredientPanel = new JPanel();
		setIngredientPanel.setLayout(new BoxLayout(setIngredientPanel, BoxLayout.X_AXIS));
		
		Vector ingredientVector = new Vector();
		try{
			ResultSet rs = database.query("SELECT ID, Name FROM Ingredient ORDER BY Name;");
			while (rs.next())ingredientVector.add(new Ingredient(rs.getInt(1),
														rs.getString(2)));
		} catch (SQLException se) {
			System.out.println("Unable to get ingredients from database. Error:");
			se.printStackTrace();
		}
		ingredientBox = new JComboBox(ingredientVector);
		ingredientBox.setMinimumSize( new Dimension( 180, 25) );
		ingredientBox.setMaximumSize( new Dimension( 180, 25) );
		ingredientBox.setPreferredSize( new Dimension( 180, 25) );
		setIngredientPanel.add(ingredientBox);
		
		setIngredientPanel.add( Box.createRigidArea(new Dimension(6,0)) );
		amountField = new JTextField();
		setIngredientPanel.add(amountField);
		
		// get availabe units from database
		Vector units = new Vector();
		try{
			ResultSet rs = database.query("SELECT ID, ABBRV FROM UNIT ORDER BY ABBRV ASC;");
			while (rs.next()){
				units.add(new eShopperUnit(rs.getInt("ID"), database));
			}
		}
		catch (SQLException se) {
			System.out.println("Unable to get units from database. Error:");
			se.printStackTrace();
		}
		unitBox = new JComboBox(units);
		setIngredientPanel.add(unitBox);
		setIngredientPanel.setMinimumSize( new Dimension(280, 25) );
		setIngredientPanel.setMaximumSize( new Dimension(280, 25) );
		setIngredientPanel.setPreferredSize( new Dimension(280, 25) );
		ingredientPanel.add(setIngredientPanel);
		
		// ingredient button panel
		buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout(buttonPanel, BoxLayout.X_AXIS) );
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		buttonPanel.add(Box.createHorizontalGlue());
		aButton = new JButton("Add");
		aButton.addActionListener(this);
		aButton.setActionCommand("addIngredient");
		buttonPanel.add(aButton);
		buttonPanel.add( Box.createRigidArea(new Dimension(10,0)) );
		aButton = new JButton("Create Ingredient");
		aButton.addActionListener(this);
		aButton.setActionCommand("createIngredient");
		aButton.setPreferredSize( new Dimension( 150, 25) );
		buttonPanel.add(aButton);
		ingredientPanel.add(buttonPanel);
		
		//make datePanel
		datePanel = new JPanel();
		datePanel.setMinimumSize( new Dimension(230, 180) );
		datePanel.setMaximumSize( new Dimension(230, 180) );
		datePanel.setPreferredSize( new Dimension(230, 180) );
		datePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		datePanel.setLayout ( new BoxLayout(datePanel, BoxLayout.Y_AXIS) );
		datePanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		datePanel.setBorder(BorderFactory.createEtchedBorder());
		cal = new eShopperCalendar();
		calTable = new JTable(cal);
		calTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		calTable.setRowSelectionAllowed(false);
		calTable.setColumnSelectionAllowed(false);
		calTable.setCellSelectionEnabled(true);
		calTable.getTableHeader().setResizingAllowed(false);
		calTable.getTableHeader().setReorderingAllowed(false);
		calTable.getSelectionModel().addListSelectionListener(this);
		calTable.getColumnModel().getSelectionModel().addListSelectionListener(this);
		dateString = new JLabel( "Item assigned to: " + cal.getLabelStringAt(calTable.getSelectedRow(), 
							calTable.getSelectedColumn()) );
		dateString.setAlignmentX(Component.CENTER_ALIGNMENT);
		datePanel.add(dateString);
		datePanel.add(calTable.getTableHeader());
		datePanel.add(calTable);
		// calendar buttons
		buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		JButton arrowButton = new JButton(createImageIcon("images/larrow.gif"));
		arrowButton.addActionListener(this);
		arrowButton.setActionCommand("previousMonth");
		buttonPanel.add(arrowButton);
		monthString = new JLabel(" " + cal.getMonthString() + " ");
		buttonPanel.add(monthString);
		arrowButton = new JButton(createImageIcon("images/rarrow.gif"));
		arrowButton.addActionListener(this);
		arrowButton.setActionCommand("nextMonth");
		buttonPanel.add(arrowButton);
		datePanel.add(buttonPanel);
 
		// toss it all on there!
		mainPanel.add(recipePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,10)) );
		mainPanel.add(ingredientPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,10)) );
		mainPanel.add(datePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,10)) );
		aButton = new JButton("Close Window");
		aButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		aButton.addActionListener(this);
		aButton.setActionCommand("close");
		mainPanel.add(aButton);
		getContentPane().add(mainPanel);
	}
	
	/**
	 * This takes the currently selected recipe in the combobox and adds
	 * it to the current list in the database.
	 *
	 */
	private void addRecipeToList() {
		list.add((Recipe)recipesBox.getSelectedItem(), null, 
			cal.getDateAt(calTable.getSelectedRow(), calTable.getSelectedColumn()));
	}
	
	/**
	 * This takes the currently selected ingredient in the combobox
	 * and adds it to the current list in the database.
	 *
	 */
	private void addIngredientToList() {
		double amount;
		try {
			amount = new Double(amountField.getText()).doubleValue();
		} catch(NumberFormatException e) {
			amount = 0;
		}
		list.add((Ingredient) ingredientBox.getSelectedItem(), null, 	cal.getDateAt(calTable.getSelectedRow(), 
			calTable.getSelectedColumn()), amount, ((eShopperUnit) unitBox.getSelectedItem()).getID());
	}
	
	/**
	 * This toggles the date assignment section between disabled and enabled.
	 *
	 */
	public void toggleDate(){
		if (dateAssigned.isSelected()){
			dateString.setEnabled(true);
			dateString.setOpaque(true);
			mealMenu.setEnabled(true);
			mealMenu.setOpaque(true);
		} else {
			dateString.setEnabled(false);
			dateString.setOpaque(false);
			mealMenu.setEnabled(false);
			mealMenu.setOpaque(false);
		}
			
	}
	
	/**
	 * This handles all of the ActionEvents from the window.
	 */
	public void actionPerformed(ActionEvent e){
		if (e.getActionCommand().equals("addRecipe"))
			addRecipeToList();
		else if (e.getActionCommand().equals("addIngredient"))
			addIngredientToList();
		else if (e.getActionCommand().equals("createRecipe"))
			displayCreateRecipeWindow();
		else if (e.getActionCommand().equals("createIngredient"))
			displayCreateIngredientWindow();
		else if (e.getActionCommand().equals("previousMonth")){
			cal.previousMonth();
			monthString.setText(cal.getMonthString());
		}
		else if (e.getActionCommand().equals("nextMonth")) {
			cal.nextMonth();
			monthString.setText(cal.getMonthString());
		}
		else if (e.getActionCommand().equals("toggle date"))
			toggleDate();
		else if (e.getActionCommand().equals("close"))
			dispose();
	}
	
	/**
	 * This handles changes in the Calendar choice.
	 */
	public void valueChanged(ListSelectionEvent e){
		dateString.setText("Item assigned to: " + cal.getLabelStringAt(calTable.getSelectedRow(), 
			calTable.getSelectedColumn()) );
	}
	
}