/*
 * Created on Apr 21, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import javax.swing.*;
import java.awt.*;
import java.sql.*;

/**
 * This window allows the user to create new ingredients and
 * insert them in the database.  The newly created ingredient will
 * also be inserted into the ingredients combobox in the
 * parent window.
 * 
 * @author Nathan
 */
public class CreateIngredientWindow extends IngredientWindow {

	JTextField nameField;
	JComboBox myIngredientBox;

	CreateIngredientWindow(eShopperList list, JComboBox ingredientBox) {
		super(list);
		this.list = list;
		this.database = list.getDB();
		myIngredientBox = ingredientBox;
		
		setTitle("eShopper: Create New Ingredient");
		
		// edit namePanel
		namePanel.setMinimumSize( new Dimension(200, 25) );
		namePanel.setMaximumSize( new Dimension(300, 25) );
		namePanel.setPreferredSize( new Dimension(300, 25) );

		namePanel.add(new JLabel("Ingredient Name:  "));
		namePanel.add(Box.createHorizontalGlue());
		nameField = new JTextField();
		namePanel.add(nameField);
		
		mainPanel.add(namePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,11)) );
		mainPanel.add(descriptionPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,17)) );
		mainPanel.add(buttonPanel);
		getContentPane().add(mainPanel);
	}
	
	/**
	 * This inserts the new ingredient into the database,
	 * and also updates the combobox on the parent window.
	 */
	public void updateDatabase(){
		
		try{
			// The database doesn't like apostrophes.
			String inputValue = nameField.getText();
			String cleanName = inputValue.replaceAll("'", "''");
			
			// First check to see if there is already an Ingredient with the
			// same name.
			String q = "SELECT Name FROM Ingredient";
			ResultSet rs = database.query(q);
			if (inResultSet(inputValue, rs)){
				JOptionPane.showMessageDialog(mainPanel,
    				    "Error: Ingredient " + rs.getString(1) + " already exists.\n" +
						"Please choose another name.",
    				    "eShopper Error",
    				    JOptionPane.ERROR_MESSAGE);
    			return;
			}
			
			database.update("INSERT INTO Ingredient (ID, Name, Description) VALUES ("
				+ null + ", '" + cleanName + "', '"
				+ descArea.getText().replaceAll("'", "''") + "');");
				
			// The "CALL IDENTITY();" SQL command returns the last IDENTITY
			// value created by the Connection
			rs = database.query("CALL IDENTITY()");
			rs.next();
			int ingredientID = rs.getInt(1);
			rs.close();
			
			//	Update combobox in parent window, if it exists.
				 if (myIngredientBox != null){
					 myIngredientBox.addItem(new Ingredient(ingredientID, nameField.getText()));
					 myIngredientBox.setSelectedIndex(myIngredientBox.getItemCount() - 1);	
				 }
			
			dispose();
		} catch (SQLException se) {
			System.out.println("Could not write to database.  Error: ");
			se.printStackTrace();
		}
		
		
	}
}
