/*
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import javax.swing.*;
import java.awt.*;
import java.sql.*;

/**
 * This is the window by which recipes are added to the database.
 * 
 * @author Nathan
 *
 */
public class CreateRecipeWindow extends RecipeWindow{

	JTextField nameField;
	JComboBox myRecipesBox;

	public CreateRecipeWindow(eShopperList list, JComboBox recipesBox){
		super(list);
		this.list = list;
		this.database = list.getDB();
		myRecipesBox = recipesBox;
		
		setTitle("eShopper: Create New Recipe");
		setSize(400, 480);
		
		namePanel.setMinimumSize( new Dimension(300, 25) );
		namePanel.setMaximumSize( new Dimension(320, 25) );
		namePanel.setPreferredSize( new Dimension(320, 25) );

		namePanel.add(new JLabel("Enter Recipe Name: "));
		nameField = new JTextField();
		namePanel.add(nameField);
		
		mainPanel.add(namePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(ingredientsPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(descriptionPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(instructionsPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,17)) );
		mainPanel.add(buttonPanel);
		getContentPane().add(mainPanel);

	}
	
	/**
	 * This saves the information displayed in the window to the database.
	 */
	public void updateDatabase(){
		try{
			// The database doesn't like apostrophes.
			String inputValue = nameField.getText();
			String cleanName = inputValue.replaceAll("'", "''");
			
			// First check to see if there is already a recipe with the
			// same name.
			String q = "SELECT Name FROM Recipe";
			ResultSet rs = database.query(q);
			if (inResultSet(inputValue, rs)){
				JOptionPane.showMessageDialog(mainPanel,
    				    "Error: Recipe " + rs.getString(1) + " already exists.\n" +
						"Please choose another name.",
    				    "eShopper Error",
    				    JOptionPane.ERROR_MESSAGE);
    			return;
			}
			
			//update Recipe table
			database.update("INSERT INTO Recipe (ID, Name, Description, Directions) VALUES ("
				+ null + ", '" + cleanName + "', '" 
				+ descArea.getText().replaceAll("'", "''") + "', '"
				+ instructionsArea.getText().replaceAll("'", "''") + "');");
			
			// The "CALL IDENTITY();" SQL command returns the last IDENTITY
			// value created by the Connection
			rs = database.query("CALL IDENTITY()");
			rs.next();
			int recipeID = rs.getInt(1);
			rs.close();
			
			//update Has table
			for (int i = 0; i < ingredientVector.size(); i++){
				RecipeIngredient ingredient = (RecipeIngredient)ingredientVector.get(i);
				database.update("INSERT INTO Has (RecipeID, IngredientID, Amount, UnitID) VALUES ("
					+ recipeID + ", " + ingredient.getID() + ", " + ingredient.getAmount() + ", '"
					+ ingredient.getUnit().getID() + "');");
			}
			
			// Update combobox in parent window, if it exists.
			if (myRecipesBox != null){
				myRecipesBox.addItem(new Recipe(recipeID, nameField.getText()));
				myRecipesBox.setSelectedIndex(myRecipesBox.getItemCount() - 1);
			}
		}
		catch (SQLException se){
			System.out.println("Error: CreateRecipeWindow::updateDatabase");
			se.printStackTrace();
		}
		dispose();
	}
}