/*
 * Created on May 14, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.awt.event.ActionEvent;

import javax.swing.*;
import java.sql.*;
import java.util.*;

/**
 *	 
 *
 * @author npe2
 *
 */
public class ManageListsWindow extends eShopperGUI {

	JPanel mainPanel, buttonPanel;
	JScrollPane listPane;
	eShopperList list;
	JList listList;
	JButton aButton;
	MainWindow myMainWindow;

	public ManageListsWindow(eShopperList list, MainWindow receivedMainWindow) {
		this.database = list.getDB();
		this.list = list;
		
		setTitle("eShopper: Manage Lists");
		setSize(400, 200);
		setResizable(false);
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		
		listPane = new JScrollPane();
		listPane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createEmptyBorder(10,10,10,10),
			BorderFactory.createEtchedBorder()));
		listList = new JList(getLists());
		listPane.getViewport().add(listList);
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		aButton = new JButton("Delete");
		aButton.addActionListener(this);
		aButton.setActionCommand("Delete");
		buttonPanel.add(aButton);
		aButton = new JButton("Rename");
		aButton.addActionListener(this);
		aButton.setActionCommand("Rename");
		buttonPanel.add(aButton);
		aButton = new JButton("Close Window");
		aButton.addActionListener(this);
		aButton.setActionCommand("Close");
		buttonPanel.add(aButton);
		
		mainPanel.add(listPane);
		mainPanel.add(buttonPanel);
		getContentPane().add(mainPanel);
		
		myMainWindow = receivedMainWindow;
	}

	/**
	 * This gets the lists in the database and puts them in a vector.
	 * 
	 * @return the vector of lists
	 */
	private Vector getLists(){
		Vector v = new Vector();
		
		try{
					ResultSet rs = database.query("SELECT ID, Name FROM List WHERE ID > -1;");
					while (rs.next()){
						v.add(new List(rs.getInt("ID"), rs.getString("Name")));
					}
					rs.close();
				} catch (SQLException se){
					System.out.println("Could not get lists from database.  Error:");
					se.printStackTrace();
				}
		return v;
	}

	/**
	 * This handles all of the ActionEvents generated by the buttons.
	 */
	public void actionPerformed(ActionEvent e) {
		if(listList.getSelectedValue() != null) {
				if (e.getActionCommand().equals("Rename")) {
				String listName;
				listName = JOptionPane.showInputDialog(this, "Enter a name for your list:", ((List) listList.getSelectedValue()).getName());
				String cleanName;
				if (listName != null){
						cleanName = listName.replaceAll("'", "''");
				} else return;
				// Check to see if there is already a list with that name
				try {
					ResultSet rs = database.query( 	"SELECT Name FROM List");
					if (inResultSet(listName, rs)){
							JOptionPane.showMessageDialog(null,
							"Error: List " + rs.getString(1) + " already exists.",
							"eShopper Error",
							JOptionPane.ERROR_MESSAGE);
					} else {
						((List) listList.getSelectedValue()).rename(listName, database);
						listList = new JList(getLists());
						listPane.getViewport().add(listList);
						list.update();
						myMainWindow.listSubMenu.removeAll();
						myMainWindow.createListMenu();
					}
				} catch (SQLException e1) {
					System.out.println("Error: ManageListsWindow::actionPerformed");
					e1.printStackTrace();
				}
			}
		} 
		if (e.getActionCommand().equals("Delete")) {
			int n =0;
			if(listList.getSelectedValue() != null)  n = JOptionPane.showConfirmDialog(
							this, "Are you sure you want to delete this list?",
							"Delete Confirmation",
							JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				if(listList.getSelectedValue() != null) {
					((List) listList.getSelectedValue()).delete(database);
					listList = new JList(getLists());
					listPane.getViewport().add(listList);
					list.update();
					myMainWindow.listSubMenu.removeAll();
					myMainWindow.createListMenu();
				}
			}
		}
		
		if(e.getActionCommand().equals("Close")) {
			dispose();
		}
	}
}
