/*
 * Created on May 11, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

/**
 * This is an ingredient that is associated with a recipe.
 * 
 * @author Nathan
 *
 */
public class RecipeIngredient extends Ingredient{
	
	double myAmount;
	eShopperUnit myUnit;
	
	/**
	 * This is the only constructor, there is no default constructor.
	 * @param ID The ID of the ingredient in the database.
	 * @param name The name of the ingredient in the database.
	 * @param amount The amount of the ingredient that goes in the recipe.
	 * @param unit An eShopperUnit that the ingredient is measured in.
	 */
	RecipeIngredient(int ID, String name, double amount, eShopperUnit unit){
		super(ID, name);
		myAmount = amount;
		myUnit = unit;
	}
	
	/**
	 * This is the accessor method for the amount of the ingredient.
	 * 
	 * @return the amount of the ingredient
	 */
	public double getAmount(){
		return myAmount;
	}
	
	/**
	 * This is the accessor method for the unit the ingredient is measured in.
	 * 
	 * @return the unit of the ingredient
	 */
	public eShopperUnit getUnit(){
		return myUnit;
	}
	
	/**
	 * Overloads the toString() method, returning the name, amount and
	 * units of the ingredient.
	 * 
	 * @return A formatted String with the name, amount and units in it.
	 */
	public String toString(){
		return myName + " -- " + myAmount + " " + myUnit;
	}
	
	/**
	 * Overloads the equals() method.
	 * 
	 * @return true if the IDs of the two ingredients are equal; otherwise false.
	 */
	public boolean equals(Object anotherIngredient){
		return this.getID() == ((RecipeIngredient)anotherIngredient).getID()
			& this.getAmount() == ((RecipeIngredient)anotherIngredient).getAmount()
			& this.getUnit().getID() == ((RecipeIngredient)anotherIngredient).getUnit().getID();
	}
}