/*
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.sql.*;

/**
 * This is the parent of the windows to create and manage recipes.
 * 
 * @author Nathan
 */
abstract public class RecipeWindow extends eShopperGUI {
	
		JPanel mainPanel = new JPanel();
		JPanel namePanel = new JPanel();
		JPanel ingredientsPanel = new JPanel();
		JPanel instructionsPanel = new JPanel();
		JTextArea instructionsArea;
		JScrollPane instructionsScrollPane;
		JPanel descriptionPanel = new JPanel();
		JTextArea descArea;
		JScrollPane descScrollPane;
		JPanel buttonPanel = new JPanel();
		Vector ingredientVector;
		JList ingredientList;
		JComboBox unitBox;
		JTextField amountField;
	
		RecipeWindow(eShopperList list){
			this.list = list;
			this.database = list.getDB();
			
			
			setSize(400, 500);
			setResizable(false);
			mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
			mainPanel.setBorder(BorderFactory.createEmptyBorder(12,12,11,11));
		
			//
			// name panel
			//
		
			namePanel.setVisible(true);
			namePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			namePanel.setLayout ( new BoxLayout(namePanel, BoxLayout.X_AXIS) );
			//namePanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		
		
			//
			//ingredientsPanel
			//
		
			ingredientsPanel.setMinimumSize( new Dimension(300, 120) );
			ingredientsPanel.setMaximumSize( new Dimension(Integer.MAX_VALUE, 200) );
			ingredientsPanel.setPreferredSize( new Dimension(Integer.MAX_VALUE, 200) );
			ingredientsPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			ingredientsPanel.setBorder(BorderFactory.createEtchedBorder());
			ingredientsPanel.setLayout ( new BoxLayout(ingredientsPanel, BoxLayout.Y_AXIS) );
			ingredientsPanel.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(), "Ingredients"));
		
			ingredientVector = new Vector();
			ingredientList = new JList(ingredientVector);
			JScrollPane aScrollPane = new JScrollPane();
			aScrollPane.setAlignmentX(Component.CENTER_ALIGNMENT);
			aScrollPane.setBorder(BorderFactory.createEtchedBorder());
			aScrollPane.setMinimumSize( new Dimension(320, 100) );
			aScrollPane.setMaximumSize( new Dimension(320, 100) );
			aScrollPane.setPreferredSize( new Dimension(320, 100) );
			aScrollPane.getViewport().add(ingredientList);
			ingredientsPanel.add(aScrollPane);
			ingredientsPanel.add(new JLabel("Select an ingredient to add:"));
		
			// Create a panel to add ingredients to the recipe
			JPanel addIngredientPanel = new JPanel();
			addIngredientPanel.setLayout(new BoxLayout(addIngredientPanel, BoxLayout.X_AXIS));
			Vector ingredientVector = new Vector();
			
			// Populate ingredient list from database
			try{
				ResultSet rs = database.query("SELECT ID, Name FROM Ingredient ORDER BY Name;");
				while (rs.next()){
					ingredientVector.add(new Ingredient(rs.getInt(1), rs.getString(2)));
				}
			}
			catch (SQLException se) {
				System.out.println("Unable to get ingredients from database. Error:");
				se.printStackTrace();
			}
			ingredientBox = new JComboBox(ingredientVector);
			ingredientBox.setMaximumSize(new Dimension(150,30));
			ingredientBox.setMinimumSize(new Dimension(150,30));
			ingredientBox.setPreferredSize(new Dimension(150,30));
			addIngredientPanel.add(ingredientBox);
			addIngredientPanel.add( Box.createRigidArea(new Dimension(6,0)) );
			amountField = new JTextField();
			addIngredientPanel.add(amountField);
			
			// get availabe units from database
			Vector units = new Vector();
			try{
				ResultSet rs = database.query("SELECT ID, ABBRV FROM UNIT ORDER BY ABBRV ASC;");
				while (rs.next()){
					units.add(new eShopperUnit(rs.getInt(1), database));
				}
			}
			catch (SQLException se) {
				System.out.println("Unable to get units from database. Error:");
				se.printStackTrace();
			}
			unitBox = new JComboBox(units);
			addIngredientPanel.add(unitBox);
			addIngredientPanel.setMinimumSize( new Dimension(200, 25) );
			addIngredientPanel.setMaximumSize( new Dimension(320, 25) );
			addIngredientPanel.setPreferredSize( new Dimension(320, 25) );
			ingredientsPanel.add(addIngredientPanel);
			ingredientsPanel.add( Box.createRigidArea(new Dimension(0,5)) );
			
			// add create and remove ingredient buttons
			JPanel ingredientButtonPanel = new JPanel();
			ingredientButtonPanel.setLayout(new BoxLayout(ingredientButtonPanel, BoxLayout.X_AXIS));
			ingredientButtonPanel.setMinimumSize( new Dimension(200, 25) );
			ingredientButtonPanel.setMaximumSize( new Dimension(320, 25) );
			ingredientButtonPanel.setPreferredSize( new Dimension(320, 25) );
			JButton aButton = new JButton("Add");
			aButton.addActionListener(this);
			aButton.setActionCommand("Add Ingredient");
			ingredientButtonPanel.add(aButton);
			ingredientButtonPanel.add( Box.createRigidArea(new Dimension(6,0)) );
			aButton = new JButton("Remove");
			aButton.addActionListener(this);
			aButton.setActionCommand("Remove Ingredient");
			ingredientButtonPanel.add(aButton);
			ingredientButtonPanel.add(Box.createHorizontalGlue());
			aButton = new JButton("Create New...");
			aButton.addActionListener(this);
			aButton.setActionCommand("Create Ingredient");
			ingredientButtonPanel.add(aButton);
			ingredientsPanel.add(ingredientButtonPanel);
			ingredientsPanel.add( Box.createRigidArea(new Dimension(6,0)) );
		
		
			//
			//descriptionPanel
			//
		
			descriptionPanel.setMinimumSize( new Dimension(320, 70) );
			descriptionPanel.setMaximumSize( new Dimension(320, 70) );
			descriptionPanel.setPreferredSize( new Dimension(320, 70) );
			descriptionPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			descriptionPanel.setLayout(new BoxLayout(descriptionPanel, BoxLayout.Y_AXIS));
		
			descriptionPanel.add(new JLabel("Description:"));
			descArea = new JTextArea(3, 20);
			descArea.setWrapStyleWord(true);
			descArea.setLineWrap(true);
			descScrollPane = new JScrollPane(descArea);
			descScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			descScrollPane.setPreferredSize(new Dimension(200, 50));
			descriptionPanel.add(descScrollPane);
		
		
			//
			//instructionPanel
			//
		
			instructionsPanel.setMinimumSize( new Dimension(320, 70) );
			instructionsPanel.setMaximumSize( new Dimension(320, 70) );
			instructionsPanel.setPreferredSize( new Dimension(320, 70) );
			instructionsPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			instructionsPanel.setLayout(new BoxLayout(instructionsPanel, BoxLayout.Y_AXIS));
		
			instructionsPanel.add(new JLabel("Instructions:"));
			instructionsArea = new JTextArea(3, 20);
			instructionsArea.setWrapStyleWord(true);
			instructionsArea.setLineWrap(true);
			instructionsScrollPane = new JScrollPane(instructionsArea);
			instructionsScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			instructionsScrollPane.setPreferredSize(new Dimension(200, 50));
			instructionsPanel.add(instructionsScrollPane);
		
			//
			//buttonPanel
			//
		
			buttonPanel.setMinimumSize( new Dimension(320, 30) );
			buttonPanel.setMaximumSize( new Dimension(320, 30) );
			buttonPanel.setPreferredSize( new Dimension(320, 30) );
			buttonPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.add(Box.createHorizontalGlue());
			aButton = new JButton("Save");
			aButton.addActionListener(this);
			aButton.setActionCommand("Save");
			buttonPanel.add(aButton);
			buttonPanel.add( Box.createRigidArea(new Dimension(6,0)) );
			aButton = new JButton("Close");
			aButton.addActionListener(this);
			aButton.setActionCommand("Close");
			buttonPanel.add(aButton);

		}
		
		/**
		 * This adds the currently selected ingredient to the currently selected
		 * recipe.  An amount value is required.
		 *
		 */
		public void addIngredientToRecipe(){
			// Require an amount for the ingredient.
			if (amountField.getText().equals("")) {
				JOptionPane.showMessageDialog(mainPanel,
    				    "Amount is a required field.\nPlease enter an amount.",
    				    "eShopper Error",
    				    JOptionPane.ERROR_MESSAGE);
    			return;
			}
			RecipeIngredient ingredient = new RecipeIngredient(((Ingredient) ingredientBox.getSelectedItem() ).getID(),
																		ingredientBox.getSelectedItem().toString(),
																		(new Double(amountField.getText())).doubleValue(),
																		(eShopperUnit)unitBox.getSelectedItem());
			ingredientVector.add(ingredient);
			ingredientList.setListData(ingredientVector);
		}
		
		/**
		 * This removes the currently selected ingredient from the currently selected
		 * recipe.
		 *
		 */
		public void removeIngredientFromRecipe(){
			ingredientVector.remove(ingredientList.getSelectedValue());
			ingredientList.setListData(ingredientVector);
		}
		
		abstract public void updateDatabase();
	
		/**
		 * This handles all of the ActionEvents generated by the buttons in
		 * the window.
		 */
		public void actionPerformed(ActionEvent e){
			if (e.getActionCommand().equals("Add Ingredient"))
				addIngredientToRecipe();
			else if (e.getActionCommand().equals("Remove Ingredient"))
				removeIngredientFromRecipe();
			else if (e.getActionCommand().equals("Create Ingredient"))
				displayCreateIngredientWindow();
			else if (e.getActionCommand().equals("Save"))
				updateDatabase();
			else if (e.getActionCommand().equals("Close"))
				dispose();
		}
}
