/*
 * Created on May 12, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

/**
 * This is a buffer that holds the actions the user performs
 * and provides Undo and Redo methods.
 * 
 * @author Adam
 */
public class UndoBuffer {
	/* Some of the following circular buffer code is heavily inspired
	 * by/borrowed from John Morris's lab example found at:
	 * http://ciips.ee.uwa.edu.au/~morris/External/Java/Labs/Threads.html
	 * 
	 */
	// Counts the number of data values in the buffer
	private static final int MAX_SIZE = 8;

	// Counts entry within the circular buffer where the oldest value
	// is stored.
	private int base = 0;

	// Stores the data values
	private eShopperAction[] circBuffer;

	// Counts the number of data values in the buffer
	private int size = 0;
	
	// Constructor
	public UndoBuffer(){
		circBuffer = new eShopperAction[MAX_SIZE];
		
	}
	
	/**
	 * This tells if the buffer is empty or not.
	 * @return true if empty; false if there is something in the buffer.
	 */
	private boolean bufferIsEmpty()
	{
		// returns true iff the circular buffer has no values in it.
	    return (size == 0);
	}

	/**
	 * This tells if the buffer is full or not.
	 * @return true if full, false if the buffer is not full.
	 */
	private boolean bufferIsFull()
	{
		// returns true iff the circular buffer has no more space.
	    return (size == MAX_SIZE);
	}

	private eShopperAction getValueFromBuffer()
	{
		// puts an extra value into the circular buffer
	    int index = base;
	    base = (base + 1) % MAX_SIZE;
	    size--;
	    return circBuffer[index];
	}

	private void putValueIntoBuffer(int value)
	{
		// puts a value from the circular buffer
	    int index = (base + size) % MAX_SIZE;
	    //circBuffer[index] = value;
	    size++;
	}
	
}
