/*
 * Created on May 12, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

import java.sql.*;

/**
 * This is a class that can represent any undoable action in
 * eShopper.  When actions are performed with eShopper, an
 * instance of this class is created and added to the undo
 * buffer.
 * 
 * @author Adam
 */
public class eShopperAction {
	// Used by all action types
	int actionPerformed;
	
	// Used by Clear List
	int clearedID, newID;
	String clearedListName, clearedListOwner;
	
	// Used by Add/Remove Item
	int myNodeID; // Unique ID for the ingredientlist/recipelist table
	int myItemID; // Unique ID for the recipe/ingredient table
	String itemDate;
	String myNodeType;
	double myNodeAmount;
	int myUnitID;
	
	public eShopperAction(){
		actionPerformed = 0;
		itemDate = "0000-00-00";
		clearedID = 0;
		myNodeID = 0;
		myNodeType = "";
		myNodeAmount = 0.0;
		myUnitID = 0;
	}
	
	/**
	 * This allows the type of action to be set.  The available
	 * types are:
	 * 1: Clear List
	 * 2: Add Item
	 * 3: Remove Item 
	 * 
	 * @param action The type of action.
	 * @param list The list upon which the action was performed.
	 */
	public void setAction(int action, eShopperList list, eShopperNode changedNode){
		actionPerformed = action;
		switch(actionPerformed){
			// Clear List
			case 1:
				clearedID = list.getListID();
				clearedListName = list.getListName();
				clearedListOwner = list.getListOwner();
				newID = -1;
				try{
					// Get the lowest existing listID
					String q = "SELECT TOP 1 ID FROM List ORDER BY ID ASC";
					ResultSet rs = list.getDB().query(q);
					if (rs.next()){
						newID = rs.getInt(1) - 1;
					}
					if (newID == 0)
						newID = -1;
					
					// First, copy the list ID into the "Hidden Undo Lists"
					// range ( ID < 0 ).
					q = "INSERT INTO List VALUES('" + newID + "', SELECT Owner FROM List where ID = '" + clearedID + "', '" + list.getListName().replaceAll("'", "''") + "')";
					list.getDB().update(q);
					
					// Then, copy all of the recipes and ingredients over
					// to the hidden list.
					q = "SELECT * FROM RecipeList WHERE ListID = '" + clearedID + "'";
					rs = list.getDB().query(q);
					while (rs.next()){
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						q = "INSERT INTO RecipeList VALUES(" +
							"null," + 
							rs.getString("RecipeID") + "," + 
							newID + "," + 
							myMeal + "," + 
							myDate + ")";
						list.getDB().update(q);
					}
					
					q = "SELECT * FROM IngredientList WHERE ListID = '" + clearedID + "'";
					rs = list.getDB().query(q);
					while (rs.next()){
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myAmount = rs.getString("Amount");
						if (myAmount != null)
							myAmount = "'" + myAmount + "'";
						q = "INSERT INTO IngredientList VALUES(" +
							"null," + 
							rs.getString("IngredientID") + "," + 
							newID + ", " + 
							myMeal + "," +
							myDate + "," +
							myAmount + "," +
							rs.getString("UnitID") + ")";
						list.getDB().update(q);
					}
				}
				catch (SQLException se){
					System.out.println("Error: eShopperAction::setAction(1)");
					se.printStackTrace();
				}
				break;
			
			// Add Item
			case 2:
				myNodeID = changedNode.nodeID();
				itemDate = changedNode.getDate();
				System.out.println(changedNode.nodeID() + " " + changedNode.getDate());
				if (changedNode instanceof eShopperRecipe){
					System.out.println("Checkpoint");
					myNodeType = "eShopperRecipe";
					myItemID = ((eShopperRecipe)changedNode).recipeID();
				}else if (changedNode instanceof eShopperIngredient){
					myNodeType = "eShopperIngredient";
					myItemID = ((eShopperIngredient)changedNode).ingredientID();
					myNodeAmount = ((eShopperIngredient)changedNode).getQuantity();
					myUnitID = ((eShopperIngredient)changedNode).getUnitID();
				}
				break;
				
			// Remove Item
			case 3:
				myNodeID = changedNode.nodeID();
				itemDate = changedNode.getDate();
				if (changedNode instanceof eShopperRecipe){
					myNodeType = "eShopperRecipe";
					myItemID = ((eShopperRecipe)changedNode).recipeID();
				}else if (changedNode instanceof eShopperIngredient){
					myNodeType = "eShopperIngredient";
					myItemID = ((eShopperIngredient)changedNode).ingredientID();
					myNodeAmount = ((eShopperIngredient)changedNode).getQuantity();
					myUnitID = ((eShopperIngredient)changedNode).getUnitID();
				}
				break;
		}
	}
	
	/**
	 * This returns the type of action.  The types that can be
	 * returned are:
	 * 1: Clear List
	 * 2: Add Item
	 * 3: Remove Item
	 * 
	 * @return An integer representing the type of action performed.
	 */
	public int getAction(){
		return actionPerformed;
	}
	
	/**
	 * This reverses the affects of the action.
	 *
	 */
	public void undoAction(eShopperList list){
		switch(actionPerformed){
			// Clear List
			case 1:
				if (clearedID == 0)
					return;
				try{
					// First, see if the cleared list already exists.  If it doesn't,
					// create it.
					String q = "SELECT ID FROM List WHERE ID = " + clearedID;
					ResultSet rs = list.getDB().query(q);
					
					if (!rs.next()){
						q = "INSERT INTO List VALUES('" + clearedID + "', '" + clearedListOwner.replaceAll("'", "''") + "', '" + clearedListName.replaceAll("'", "''") + "')";
						list.getDB().update(q);
					}
					
					// Then, copy all of the recipes and ingredients over
					// from the hidden list.
					q = "SELECT * FROM RecipeList WHERE ListID = '" + newID + "'";
					rs = list.getDB().query(q);
					while (rs.next()){
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						q = "INSERT INTO RecipeList VALUES(" +
							"null," + 
							rs.getString("RecipeID") + "," + 
							clearedID + "," + 
							myMeal + "," + 
							myDate + ")";
						list.getDB().update(q);
					}
					
					q = "SELECT * FROM IngredientList WHERE ListID = '" + newID + "'";
					rs = list.getDB().query(q);
					while (rs.next()){
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myAmount = rs.getString("Amount");
						if (myAmount != null)
							myAmount = "'" + myAmount + "'";
						q = "INSERT INTO IngredientList VALUES(" +
							"null," + 
							rs.getString("IngredientID") + "," + 
							clearedID + "," + 
							myMeal + "," +
							myDate + "," +
							myAmount + "," +
							rs.getString("UnitID") + ")";
						list.getDB().update(q);
					}
					list.update();
					removeHiddenList(newID, list);
				}
				catch (SQLException se){
					System.out.println("Error: eShopperAction::undoAction(1)");
					se.printStackTrace();
				}
				break;
			
			// Add Item
			case 2:
				if (myNodeID > 0){
					try{
						if (myNodeType.equals("eShopperRecipe")){
							String q = "DELETE FROM RecipeList WHERE ID = " + myNodeID;
							boolean result = list.getDB().update(q);
						} else if (myNodeType.equals("eShopperIngredient")){
							String q = "DELETE FROM IngredientList WHERE ID = " + myNodeID;
							boolean result = list.getDB().update(q);
						}
					}
					catch (SQLException se){
						System.out.println("Error: eShopperAction::undoAction(2)");
						se.printStackTrace();
					}
				list.update();
				}
				break;
				
			// Remove Item
			case 3:
				if (myNodeID > 0){
					try{
						String myNewDate = itemDate;
						if (myNewDate != null)
							myNewDate = "'" + myNewDate + "'";
						
						if (myNodeType.equals("eShopperRecipe")){
							String q = "INSERT INTO RecipeList VALUES(" +
								null + "," +
								myItemID + "," +
								list.getListID() + "," +
								null + "," +
								myNewDate + ")";
							boolean result = list.getDB().update(q);
						} else if (myNodeType.equals("eShopperIngredient")){
							String q = "INSERT INTO IngredientList VALUES(" +
								null + "," +
								myItemID + "," +
								list.getListID() + "," +
								null + "," +
								myNewDate + "," +
								myNodeAmount + "," +
								myUnitID + ")";
							boolean result = list.getDB().update(q);
						}
					}
					catch (SQLException se){
						System.out.println("Error: eShopperAction::undoAction(3)");
						se.printStackTrace();
					}
				list.update();
				}
				break;
		}
	}
	
	/**
	 * This sets the node the action was performed on (if applicable).  
	 * 
	 * @param node The eShopper node upon which the action was performed.
	 */
	public void setNode(eShopperNode node){
		// Make sure the action has a node associated with it.
		if (actionPerformed > 2){
			myNodeID = node.nodeID();
		}
	}
	
	/**
	 * This returns the ID of the node the action was performed on (if applicable).
	 * 
	 * @return The ID of the node the action was performed on, if it exists; otherwise,
	 * 0.
	 */
	public int getNodeID(){
		// Make sure the action has a node associated with it.
		if (actionPerformed > 2){
			return myNodeID;
		}else{
			return 0;
		}
	}
	
	/**
	 * This returns the date associated with the Action's node (if applicable).
	 *
	 * @return The date associated with the Action's node, if it exists;
	 * otherwise, null.
	 */
	public String getDate(){
		// Make sure the action has a date associated with it.
		if (actionPerformed > 2){
			return itemDate;
		} else {
			return null;
		}
	}
	
	/**
	 * This removes the specified hidden list.
	 *
	 * @param listID The ID of the list to be removed.
	 *
	 */
	private void removeHiddenList(int listID, eShopperList list){
		try{
			// Delete the list from the lists panel.
			String q = "DELETE FROM List WHERE ID = " + listID;
			boolean result = list.getDB().update(q);
			
			// Delete all recipes in the list
			q = "DELETE FROM RecipeList WHERE ListID = " + listID;
			result = list.getDB().update(q);
			
			// Delete all ingredients in the list
			q = "DELETE FROM IngredientList WHERE ListID = " + listID;
			result = list.getDB().update(q);
		}
		catch (SQLException se){
			System.out.println("Error: eShopperAction::undoAction(1)");
			se.printStackTrace();
		}
	}
	
}
