/*
 * eShopperClipboard.java
 * 
 * Created: May 5, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.sql.*;

/**
 * This class allows for copying and pasting within eShopper.
 * 
 * @author Adam Goforth
 *
 */
public class eShopperClipboard {
	
	// Yay for polymorphism.  The clipboard can hold
	// any type of eShopperNode.
	eShopperNode currentItem;
	String currentDate;
	eShopperDB database;
	
	/** 
	 * This is the constructor.
	 * 
	 * @param db The eShopperDB that holds the shopping lists.
	 */
	public eShopperClipboard(eShopperDB db){
		database = db;
		currentDate = new String();
	}
	
	/**
	 * This returns the item currently in the clipboard.
	 * @return The item currently in the clipboard.
	 */
	public eShopperNode getCurrentItem(){
		return currentItem;
	}
	
	/**
	 * This adds an item to the clipboard, to later be retrieved.
	 * 
	 * @param newNode The item that should be added to the clipboard.
	 */
	public void add(eShopperNode newNode) {
		if ( newNode.nodeID() > 0) {
			currentItem = newNode;
			currentDate = currentItem.getDate();
		}
	}
	
	/**
	 * This takes the currently selected item from the list, and adds it to
	 * the clipboard, then removes it from the list.
	 * 
	 * @param list The list from which the item should be taken.
	 */
	public void cut(eShopperList list){
		// Make sure something is selected.
		if (list.getSelectionCount() > 0){
			eShopperNode selectedNode = (eShopperNode) list.getSelectionPath().getLastPathComponent();
			// Make sure what is selected can be copied.
			if ( selectedNode.nodeID() > 0) {
				// Order is essential here!
				this.add( (eShopperNode) list.getSelectionPath().getLastPathComponent() );
				list.remove();
			}
		}
	}
	
	/**
	 * This takes the item currently in the clipboard and inserts it
	 * into the selected place in the list.
	 * 
	 * @param list The list into which the clipboard item should be inserted.
	 */
	public void paste(eShopperList list) {
		
		String newDate = new String("null");

		
		// Make sure something is selected.
		if (list.getSelectionCount() > 0){
			eShopperNode selectedNode = (eShopperNode) list.getSelectionPath().getLastPathComponent();
			// Make sure the selected thing has a Date.
			if (!(selectedNode.getDate() == null) && !(selectedNode.getDate().equals("")) ) {
				newDate = "'" + selectedNode.getDate() + "'";
			}
		}
		
		//	Each type of node must be handled differently.
		if (currentItem instanceof eShopperRecipe){
			eShopperRecipe currentRecipe = (eShopperRecipe) currentItem;
			String q = new String();
			int recipeid;
			
			try {
				recipeid = currentRecipe.recipeID();
				
				// Insert the Node into the database and refresh the view.
				q = "INSERT INTO RECIPELIST VALUES(" + null + ", " + recipeid + ", " + list.getListID() 
					+ ", " + null + ", " + newDate + ")";
				database.update(q);
				list.update();
			} catch (SQLException e) {
				System.out.println("Error: eShopperClipboard::paste (Recipe)");
				e.printStackTrace();
			}
		} // end if (currentItem instanceof eShopperRecipe)
		
		if (currentItem instanceof eShopperIngredient){
			eShopperIngredient currentIngredient = (eShopperIngredient) currentItem;
			String q = new String();
			int ingredientid;
			
			try {
				ingredientid = currentIngredient.ingredientID();
				
				// Insert the Node into the database and refresh the view.
				q = "INSERT INTO INGREDIENTLIST VALUES(" + null + ", " + ingredientid + ", " + list.getListID() 
					+ ", " + null + ", " + newDate + ", " + currentIngredient.getQuantity() + ", " + currentIngredient.getUnitID() + ")";
				database.update(q);
				list.update();
			} catch (SQLException e) {
				System.out.println("Error: eShopperClipboard::paste (Ingredient)");
				e.printStackTrace();
			}
		} // end if (currentItem instanceof eShopperIngredient)
	} // end paste()
	
} // end class eShopperClipboard
