/*
 * Created on Apr 21, 2004
 *
 * eShopperDB.java
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

import java.sql.*;

/**
 * This class provides database access to eShopper.  When eShopper is started,
 * an instance of an eShopperDB should be created, and all database interaction
 * should go through that instance.
 * This uses hsqldb (http://hsqldb.sourceforge.net/) for the database, and the
 * hsqldb.jar must be in the classpath or in the eShopper jar.
 * The eShopper database files must be in the db subdirectory of eShopper.
 * 
 * @author Adam Goforth
 *
 */
public class eShopperDB {
	
	// Use a single connection for the class.
	Connection conn;
	
	// Form a database connection
	public eShopperDB() {
		
	// Load the HSQL Database Engine JDBC driver
        // hsqldb.jar should be in the class path or made part of the eShopper jar
 		try {
			Class.forName("org.hsqldb.jdbcDriver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		// Connect to the eShopper database files
		try {
			conn = DriverManager.getConnection ("jdbc:hsqldb:db/eShopper", "sa", "");
		} catch (SQLException e1) {
			e1.printStackTrace();
		}

	} // End constructor
	

   /**
	*  If there are no other open connections,
	* the database is written out to files and shuts down.
	* This happens anyway at garbage collection
	* when eShopper ends.
	*/
	public void shutdown() throws SQLException {
	
		conn.close();

	} // End shutdown()
    
	/**  
	 * This method is used for executing the SQL commands DROP, INSERT and UPDATE.
	 * 
	 * @param expression The SQL command to be executed.
	 * @return true if the query ran correctly, false if not.
	 * @throws SQLException If there is a database error.
	 */
	public synchronized boolean update(String expression) throws SQLException {

		Statement st = null;

		st = conn.createStatement();

		// Run the query
		int i = st.executeUpdate(expression);

		// Close the Statement
		st.close();
        
		// Return false if the query didn't run correctly
		if (i == -1) {
			return false;
		} // else return true
		return true;

	} // End update()
	
	/**
	 * This method is used for executing the SQL commands CREATE and SELECT.
	 * 
	 */
	public synchronized ResultSet query(String expression) throws SQLException {
		
		Statement st = null;
		ResultSet rs = null;

		st = conn.createStatement();  
		rs = st.executeQuery(expression); 

		// If the Statement gets closed explicitly, the ResultSet is invalidated.
		// We'll let it happen automatically at the end of the method.
		//st.close();
		
		// Return the results of the query
		return rs;

	} // End query()
	
} // End class eShopperDB
