/*
 * Created on Apr 29, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

import java.awt.event.ActionListener;
import javax.swing.*;
import java.sql.*;


/**
 * All the GUI objects (i.e. windows) inheirit from this class and it contains 
 * the function calls to start any of the other windows, as well as the list 
 * and database attributes they all need.
 * 
 * @author Jim Laing
 */
public abstract class eShopperGUI extends JFrame implements ActionListener  {

	eShopperList list;
	eShopperDB database;
	JComboBox ingredientBox, recipesBox;
	
	/**
	 * This launches the AddItemWindow.
	 *
	 */
	protected void displayAddItemWindow(){
		AddItemWindow anAddItemWindow = new AddItemWindow(list);
		anAddItemWindow.setLocationRelativeTo(this);
		anAddItemWindow.setVisible(true);
	}
	
	/**
	 * This launches the ManageRecipesWindow.
	 *
	 */
	protected void displayManageRecipesWindow(){
		ManageRecipesWindow aManageRecipesWindow = new ManageRecipesWindow(list);
		aManageRecipesWindow.setLocationRelativeTo(this);
		aManageRecipesWindow.setVisible(true);
	}
	
	/**
	 * This launches the ManageIngredientsWindow.
	 *
	 */
	protected void displayManageIngredientsWindow(){
		ManageIngredientsWindow aManageIngredientsWindow = new ManageIngredientsWindow(list);
		aManageIngredientsWindow.setLocationRelativeTo(this);
		aManageIngredientsWindow.setVisible(true);
	}
    
	/**
	 * This launches the CreateRecipeWindow.
	 *
	 */
	protected void displayCreateRecipeWindow(){
		CreateRecipeWindow aCreateRecipeWindow = new CreateRecipeWindow(list, recipesBox);
		aCreateRecipeWindow.setLocationRelativeTo(this);
		aCreateRecipeWindow.setVisible(true);
	}
	
	/**
	 * This launches the CreateIngredientWindow.
	 *
	 */
	protected void displayCreateIngredientWindow(){
		CreateIngredientWindow aCreateIngredientWindow = new CreateIngredientWindow(list, ingredientBox);
		aCreateIngredientWindow.setLocationRelativeTo(this);
		aCreateIngredientWindow.setVisible(true);
	}
	
	/**
	 * This returns true if and only if the name is in the result set;
	 * the comparison is not case-sensitive;
	 * @param a name
	 * @param a result set
	 * @return 
	 */
	protected boolean inResultSet(String name, ResultSet rs){
		System.out.println("here");
		boolean found = false;
		try{
			while(rs.next()){
				if (rs.getString(1).toLowerCase().equals(name.toLowerCase())){
					found = true;
					break;
				}
			}
		}
		catch (SQLException se){
			System.out.println("Couldn't read from result set");
			se.printStackTrace();
		}
		return found;
	}
	
	/** 
	 * This creates an ImageIcon from an image given in the path.
	 * 
	 * @param path The path of the image to be put in the ImageIcon.
	 * @return The ImageIcon, or null if the path was invalid.
	 */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = eShopper.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}
