/*
 * Created on Apr 26, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.util.Vector;

/**
 * An abstract class to specify the functions and attributes common 
 * to all of the nodes on our eShopperList. Subclassed by eShopperRoot,
 * eShopperDate, eShopperRecipe, and eShopperIngredient.
 * 
 * @author Jim Laing
 *
 */
public abstract class eShopperNode {
	eShopperDB db;
	int listID, nodeID;
	
	/**
	 * Gets a vector of children 
	 * @return the children of a node
	 */
	public abstract Vector getChildren();
	
	/**
	 * Gets the String which is used to represent the node in a JTree
	 * @return a String representation of a node
	 */
	public abstract String toString();
	
	/**
	 * Returns any date associated with this node (i.e. any date ancestor)
	 * @return a string representation of the date
	 */
	public abstract String getDate();
	
	/**
	 * Used to get the unique ID of list items
	 * @return an int, the database of the node if it is a recipe or ingredient, 
	 * otherwise 0
	 */
	public abstract int nodeID();
}
