/*
 * Created on Apr 25, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
 package eShopper;

import java.sql.*;
import java.util.Vector;

/**
 * eShopperRecipe has eShopperIngredients as children. 
 * It is a subclass of eShopperNode.
 * 
 * @author Jim Laing
 */
public class eShopperRecipe extends eShopperNode {
		
	int recipeID;
	int nodeID;
	private Vector children;

	public eShopperRecipe(int nodeID, int recipeID, int listID, eShopperDB db) {
		this.nodeID = nodeID;
		this.recipeID = recipeID;
		this.listID = listID;
		this.db = db;
		this.children = getIngredientVector();
	}

	/**
	 * This returns the vector of the children of the recipe.
	 * 
	 * @return the children of the recipe
	 */
	public Vector getChildren() {
		return children;
	}
	
	public String toString() {
		ResultSet rs;
		try {
			rs = db.query("SELECT NAME FROM RECIPE WHERE ID = " + recipeID);
			rs.next();
			return rs.getString("NAME");
		} catch (SQLException e) {
			System.out.println("Error: eShopperRecipe::toString");
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * The unique ID of this recipe associated with this list
	 * @return an int representing this node in the recipelist table
	 */
	public int nodeID(){ return nodeID; }
	
	/**
	 * Used to get the ID which identifies this recipe
	 * @return the ID of this recipe in the recipe table
	 */
	public int recipeID(){ return recipeID; }
	
	/**
	 * This returns the date associated with the recipe.
	 * 
	 * @return A string representation of the date, or an empty string if it is unassigned.
	 */
	public String getDate(){
		String myDate = "";
		ResultSet rs;
		try {
			rs = db.query("SELECT DATE FROM RECIPELIST WHERE ID = " + nodeID);
			rs.next();
			myDate = rs.getString("DATE");
		} catch (SQLException e) {
			System.out.println("Error: eShopperRecipe::date");
			e.printStackTrace();
		}
		return myDate;
	}
	
	/**
	 * This returns a vector of all of the Ingredients in this Recipe.
	 * 
	 * @return A vector of the Ingredients in the recipe.
	 */
	private Vector getIngredientVector() {
			Vector v = new Vector();
			String q = new String();
		
			try {
				// get all the ingredients in a recipe
				q = "SELECT INGREDIENTID, AMOUNT, UNITID FROM RECIPE, HAS, INGREDIENT WHERE " +
					"INGREDIENT.ID = HAS.INGREDIENTID AND HAS.RECIPEID = RECIPE.ID " +
					"AND RECIPE.ID = " + recipeID;
				ResultSet rs = db.query(q);
				while(rs.next()) 
					v.add(new eShopperIngredient(0, rs.getInt("INGREDIENTID"), 
						listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));	
				return v;
			
			} catch (SQLException e) {
				System.out.println("Error: eShopperRecipe::getIngredients");
				e.printStackTrace();
			}
			return null;
	}
	
}