/*
 * Created on Apr 25, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
package eShopper;

import java.sql.*;
import java.util.Vector;

/**
 * This class takes care of all the view stuff and is consequently the most messy.
 * Other than that, it is fairly straightforward.
 * 
 * @author Jim Laing
 */

public class eShopperRoot extends eShopperNode {
	
	private int view;
	static int DATE = 0;
	static int RECIPE = 1;
	static int ITEM = 2;
	Vector dateViewChildren;
	Vector recipeViewChildren;
	Vector itemViewChildren;
	
	/**
	 * constructor
	 */
	public eShopperRoot(int view, int listID, eShopperDB db) {
		this.view = view;
		this.db = db;
		if(listID != 0 ) this.listID = listID;
		else {
			try {
				db.update("INSERT INTO LIST VALUES(1, 'nobody', '(default)')");
				listID = 1;
			} catch (SQLException e) {
				System.out.println("Error creating rootNode, no list");
				e.printStackTrace();
			}
		} 
		this.dateViewChildren = getDateViewVector();
		this.recipeViewChildren = getRecipeViewVector();
		this.itemViewChildren = getItemViewVector();
	}

	/**
	 * Gets all of the immediate children of the root node
	 * @return a vector of this node's children, based on the current set view
	 */
	public Vector getChildren() {
		if(view == DATE) return dateViewChildren;
		if(view == RECIPE) return recipeViewChildren;
		if(view == ITEM) return itemViewChildren;
		return null;
	}
	
	/**
	 * Gets the name of the list as set in the database and uses it as 
	 * the name of the root node
	 * @return a string representation of this node
	 */
	public String toString() {
		try {
			// get the name of the list set in the database
			ResultSet rs = db.query("SELECT NAME FROM LIST WHERE ID = " + listID);
			if(rs.next()) return rs.getString("NAME");
		} catch (SQLException e) {
			System.out.println("Error: eShopperRoot::toString()");
			e.printStackTrace();
		}
		return "";
	}

	/**
	 * There is never a date associated with a root node
	 * @return an empty string
	 */
	public String getDate() {
		// There is no date associated with the list.
		return "";
	}
	
	/**
	 * Gets the current view
	 * @return an int corresponding to one of the static 
	 * view values (DATE, RECIPE, or ITEM)
	 */
	public int getView() { return view; }
	
	/**
	 * Sets the current view
	 * @param view an int corresponding to one of the static 
	 * view values (DATE, RECIPE, or ITEM)
	 */
	public void setView(int view) { this.view = view; }
	
	/**
	 * A root node is uniquely IDed as a node value of -1
	 * @return always returns -1
	 */
	public int nodeID() {
		// Root node is uniquely identified by a nodeID of -1
		return -1;
	}
	
	/**
	 * Gets the children of the root node when the view is set to DATE
	 * @return a vector of children 
	 */
	public Vector getDateViewVector() {
		Vector v = new Vector();
		Vector unsorted = new Vector();
		String q = new String();
		
		try { 				
			// get all the dates used in the list
			q = "SELECT DISTINCT DATE FROM RECIPELIST WHERE DATE IS NOT " +
				"NULL AND LISTID = " + listID + " UNION SELECT DISTINCT " +
				"DATE FROM INGREDIENTLIST WHERE DATE IS NOT NULL AND " +
				"LISTID = " + listID;
			ResultSet rs = db.query(q);
			while(rs.next()) 
				v.add(new eShopperDate(rs.getDate("DATE"), listID, db));
			
			// get all the recipes unassociated with dates
			q = "SELECT ID, RECIPEID FROM RECIPELIST WHERE DATE IS NULL " +
				"AND LISTID = " + listID;
				rs = db.query(q);
			while(rs.next()) 
				unsorted.add(new eShopperRecipe(rs.getInt("ID"), rs.getInt("RECIPEID"), listID, db));
					
			// get all the ingredients unassociated with dates
			q = "SELECT ID, INGREDIENTID, AMOUNT, UNITID FROM INGREDIENTLIST WHERE DATE " +
				"IS NULL AND LISTID = " + listID;
				rs = db.query(q);
				while(rs.next())
					unsorted.add(new eShopperIngredient(rs.getInt("ID"), rs.getInt("INGREDIENTID"),
						listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));
						
			if(unsorted.size() > 0) v.add(new eShopperMisc(unsorted));
		
			return v;
					
		} catch (SQLException e) {
			System.out.println("Error: eShopperRoot::getChildren: DATE view");
			e.printStackTrace();	
		}		
		
		return null;
	}

	/**
	 * Gets the children of the root node when the view is set to RECIPE
	 * @return a vector of children
	 */
	public Vector getRecipeViewVector() {
		Vector v = new Vector();
		Vector unsorted = new Vector();
		String q = new String();
		
		try { 		
			// get all the recipes on the list
			q = "SELECT ID, RECIPEID FROM RECIPELIST WHERE LISTID = " + listID;
			ResultSet rs = db.query(q);
			while(rs.next()) 
				v.add(new eShopperRecipe(rs.getInt("ID"), rs.getInt("RECIPEID"), listID, db));
			
			// get all the ingredients on the list, add 'em to our vector
			q = "SELECT ID, INGREDIENTID, AMOUNT, UNITID FROM INGREDIENTLIST WHERE LISTID = " + listID;
			rs = db.query(q);
			while(rs.next()) 
				unsorted.add(new eShopperIngredient(rs.getInt("ID"), rs.getInt("INGREDIENTID"), 
					listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));
				
			if(unsorted.size() > 0) v.add(new eShopperMisc(unsorted));
			
			return v;
					
		} catch (SQLException e) {
			System.out.println("Error: eShopperRoot::getChildren: RECIPE view");
			e.printStackTrace();	
		}		
		return null;
	}

	/**
	 * Gets the children of the root node when the view is set to ITEM
	 * based upon the value of "combine" in the configure table
	 * @return a vector of children
	 */
	public Vector getItemViewVector() {
		if(doCombine()) return (new eShopperCombiner(listID, db, true)).getCombinedIngredients();
		else {
			String q = new String();
			Vector v = new Vector();
			try {
				// get ingredients
				q = "SELECT ID, INGREDIENTID, AMOUNT, UNITID FROM " +
					"INGREDIENTLIST WHERE LISTID = " + listID;
				ResultSet rs = db.query(q);	
				while(rs.next()) {
					v.add(new eShopperIngredient(rs.getInt("ID"), rs.getInt("INGREDIENTID"),
						listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));
				}
				// get recipe ingredients
				q = "SELECT INGREDIENTID, AMOUNT, UNITID FROM " +
				"RECIPE, HAS, INGREDIENT, RECIPELIST WHERE INGREDIENT.ID = " +
				"HAS.INGREDIENTID AND HAS.RECIPEID = RECIPE.ID AND RECIPE.ID = " +
				"RECIPELIST.RECIPEID AND RECIPELIST.LISTID = " + listID;
				rs = db.query(q);	
				while(rs.next()) {
					v.add(new eShopperIngredient(0, rs.getInt("INGREDIENTID"),
						listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID")));
				}
			} catch(SQLException e) {
				System.out.println("Error: eShopperRoot::getItemViewVector()");
				e.printStackTrace();
			}
			return v;
		}
	}
	
	/**
	 * Queries database and sees whether combining is configured
	 * @return a boolean, true if combining is desired. False otherwise
	 */
	public boolean doCombine() {
		try {
			ResultSet rs = db.query("SELECT VALUE FROM CONFIGURE WHERE NAME = 'combine'");
			if(rs.next()) return rs.getString("VALUE").equals("yes"); 
			else return false; 
		} catch (SQLException e) {
			return false;
		}
	}
	
}