/*
 * Created on Apr 25, 2004
 * 
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

package eShopper;

import java.util.Vector;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.*;

/**
 * This class implements TreeModel. I'm not sure exactly where
 * the treeModelListeners come in. If anyone could explain that
 * to me, that would be helpful.
 * 
 * @author Jim Laing
 */

public class eShopperTree implements TreeModel {
	
	private eShopperRoot rootNode;
	private Vector treeModelListeners;
	
	/**
	 * Constructs a new tree model of the shopping list
	 * @param view The initial view of the TreeModel. Corresponds to static values
	 * found in eShopperRoot: DATE, ITEM, and RECIPE
	 * @param listID The database ID of the list which is being built
	 * @param db The eShopperDB database from which to build the list
	 */
	public eShopperTree(int view, int listID, eShopperDB db){
		treeModelListeners = new Vector();
		rootNode = new eShopperRoot(view, listID, db);
	}

	/**
	 * Sets the view of the list
	 * @param view The new view. A static value from eShopperRoot: DATE, ITEM, or RECIPE
	 */
	public void setView(int view) {
		rootNode.setView(view);
		fireTreeStructureChanged();
	}
		
	/* this function copied from Olivier Berlanger's Geneology example
	 * http://java.sun.com/docs/books/tutorial/uiswing/components/tree.html
	 * it's black magic, as far as I am concerned. It does some voodoo stuff
	 * and makes the whole thing work as it should
	 */
	 /**
	  * Used to dynamically change the tree structure
	  */
	private void fireTreeStructureChanged() {
		int len = treeModelListeners.size();
		TreeModelEvent e = new TreeModelEvent(this, new Object[] {rootNode});
		for (int i = 0; i < len; i++) ((TreeModelListener)treeModelListeners.elementAt(i)).treeStructureChanged(e);
	}

	/*
	 *  Below are all the functions which implement the TreeModel interface
	 *   
	 */
	
	/**
	 * @return The current root node of the tree model
	 */
	public Object getRoot() {
		return rootNode;
	}

	/**
	 * @param parent An object representing a node in the Tree Model
	 * @return The number of children of the passed object
	 * 
	 */
	public int getChildCount(Object parent) {
		return ((eShopperNode) parent).getChildren().size();
	}
	
	/**
	 * @param node A node on the tree
	 * @return A boolean which is true if the passed node is a leaf node. False otherwise.
	 */
	public boolean isLeaf(Object node) {
		// might want to change this one ... haven't decided
		return (node instanceof eShopperIngredient);
	}
	
	
	// from Olivier Berlanger's Geneology example
	/**
	 * @param l a TreeModelListener to be added to the tree model
	 */
	public void addTreeModelListener(TreeModelListener l) {
		treeModelListeners.addElement(l);
	}
	
	// from Olivier Berlanger's Geneology example
	/**
	 * @param l a TreeModelListener to be removed from the tree model
	 */
	public void removeTreeModelListener(TreeModelListener l) {
		treeModelListeners.removeElement(l);
	}
	
	/**
	 * @param parent A node on the tree 
	 * @param index An index in the passed nodes child list
	 * @return The child node at the specified. Null if no child exists
	 */
	public Object getChild(Object parent, int index) {
		return ((eShopperNode) parent).getChildren().elementAt(index);
	}
	
	/**
	 * @param parent A parent node in the tree  
	 * @param child A child node in the tree
	 * @return an int representing the index of the child node
	 * relative to the parent
	 */
	public int getIndexOfChild(Object parent, Object child) {
		return ((eShopperNode) parent).getChildren().indexOf(child);
	}

	/**
	 * Not used by this model
	 */
	public void valueForPathChanged(TreePath pat, Object node) {
		// not used by this model
	}

}