/*
 * Created on May 14, 2004
 *
 * Copyright (c) 2004 Pam Betten, Nathan Edwards, Adam Goforth, 
 *     Jim Laing, Shahzad Qureshi
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
 
/*
 * Keeps track of which paths are suppossed to be expanded
 * 
 */

package eShopper;

import java.util.Vector;

import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.TreePath;

/**
 * This keeps track of all the paths in the tree that are set to be expanded
 * 
 * @author Jim Laing
 *
 */
public class expansionListener implements TreeExpansionListener {

	private Vector expandedPaths;
	
	/**
	 * constructor
	 *
	 */
	public expansionListener() {
		expandedPaths = new Vector();
	}

	/**
	 * Called when a node is collapsed. 
	 * @param arg0 a TreeExpansionEvent
	 */
	public void treeCollapsed(TreeExpansionEvent arg0) {
		expandedPaths.remove(arg0.getPath());
	}

	/**
	 * Called when a node is expanded
	 * @param arg0 a TreeExpansionEvent
	 */
	public void treeExpanded(TreeExpansionEvent arg0) {
		expandedPaths.add(arg0.getPath());
	}
	
	/**
	 * Gets the number of expanded paths
	 * @return an int which is the number of expanded paths
	 */
	public int size() {
		return expandedPaths.size();
	}
	
	/**
	 * Gets the path of a element
	 * @param i the desired index
	 * @return the path at that index
	 */
	public TreePath getPathAt(int i) {
		return ((TreePath) expandedPaths.elementAt(i));
	}

}
